from .blocked_number import BlockedNumberResource
from .call import CallResource
from .callback import CallbackResource
from .callcenter import CallCenterResource
from .company import CompanyResource
from .contact import ContactResource
from .department import DepartmentResource
from .event_subscription import EventSubscriptionResource
from .number import NumberResource
from .office import OfficeResource
from .room import RoomResource
from .sms import SMSResource
from .stats import StatsExportResource
from .transcript import TranscriptResource
from .user import UserResource
from .userdevice import UserDeviceResource
